@@ -20,12 +20,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #ifndef ENUM_LOOKUP_H
 #define ENUM_LOOKUP_H
 
-#include <config.h>
-#ifdef HAVE_GLIB
-#include <glib.h>
-#else
-#include <uglib.h>
-#endif
+#include "private.h"
 
 #define MAX_ENUM_LOOKUP_RESULTS 10
 
@@ -33,8 +28,8 @@ typedef struct enum_lookup_res{
 	char *sip_address[MAX_ENUM_LOOKUP_RESULTS];
 }enum_lookup_res_t;
 
-gboolean is_enum(const gchar *sipaddress, char **enum_domain);
-gint enum_lookup(const gchar *enum_domain, enum_lookup_res_t **res);
+bool_t is_enum(const char *sipaddress, char **enum_domain);
+int enum_lookup(const char *enum_domain, enum_lookup_res_t **res);
 void enum_lookup_res_free(enum_lookup_res_t *res);
 
 #endif
