@@ -29,37 +29,37 @@
 const char *linphone_online_status_to_string(LinphoneOnlineStatus ss){
 	const char *str=NULL;
 	switch(ss){
-		case LINPHONE_STATUS_ONLINE:
+		case LinphoneStatusOnline:
 		str=_("Online");
 		break;
-		case LINPHONE_STATUS_BUSY:
+		case LinphoneStatusBusy:
 		str=_("Busy");
 		break;
-		case LINPHONE_STATUS_BERIGHTBACK:
+		case LinphoneStatusBeRightBack:
 		str=_("Be right back");
 		break;
-		case LINPHONE_STATUS_AWAY:
+		case LinphoneStatusAway:
 		str=_("Away");
 		break;
-		case LINPHONE_STATUS_ONTHEPHONE:
+		case LinphoneStatusOnThePhone:
 		str=_("On the phone");
 		break;
-		case LINPHONE_STATUS_OUTTOLUNCH:
+		case LinphoneStatusOutToLunch:
 		str=_("Out to lunch");
 		break;
-		case LINPHONE_STATUS_NOT_DISTURB:
+		case LinphoneStatusDoNotDisturb:
 		str=_("Do not disturb");
 		break;
-		case LINPHONE_STATUS_MOVED:
+		case LinphoneStatusMoved:
 		str=_("Moved");
 		break;
-		case LINPHONE_STATUS_ALT_SERVICE:
+		case LinphoneStatusAltService:
 		str=_("Using another messaging service");
 		break;
-		case LINPHONE_STATUS_OFFLINE:
+		case LinphoneStatusOffline:
 		str=_("Offline");
 		break;
-		case LINPHONE_STATUS_PENDING:
+		case LinphoneStatusPending:
 		str=_("Pending");
 		break;
 		default:
@@ -68,31 +68,11 @@ const char *linphone_online_status_to_string(LinphoneOnlineStatus ss){
 	return str;
 }
 
-static int friend_data_compare(const void * a, const void * b, void * data){
+static int friend_compare(const void * a, const void * b){
 	LinphoneAddress *fa=((LinphoneFriend*)a)->uri;
 	LinphoneAddress *fb=((LinphoneFriend*)b)->uri;
-	const char *ua,*ub;
-	ua=linphone_address_get_username(fa);
-	ub=linphone_address_get_username(fb);
-	if (ua!=NULL && ub!=NULL) {
-		//printf("Comparing usernames %s,%s\n",ua,ub);
-		return strcasecmp(ua,ub);
-	}
-	else {
-		/* compare hosts*/
-		ua=linphone_address_get_domain(fa);
-		ub=linphone_address_get_domain(fb);
-		if (ua!=NULL && ub!=NULL){
-			int ret=strcasecmp(ua,ub);
-			//printf("Comparing hostnames %s,%s,res=%i\n",ua,ub,ret);
-			return ret;
-		}
-		else return -1;
-	}
-}
-
-static int friend_compare(const void * a, const void * b){
-	return friend_data_compare(a,b,NULL);
+	if (linphone_address_weak_equal (fa,fb)) return 0;
+	return 1;
 }
 
 
@@ -128,6 +108,7 @@ void __linphone_friend_do_subscribe(LinphoneFriend *fr){
 	char *friend=NULL;
 	const char *route=NULL;
 	const char *from=NULL;
+	const char *fixed_contact=NULL;
 	LinphoneProxyConfig *cfg;
 	
 	friend=linphone_address_as_string(fr->uri);
@@ -135,10 +116,16 @@ void __linphone_friend_do_subscribe(LinphoneFriend *fr){
 	if (cfg!=NULL){
 		route=linphone_proxy_config_get_route(cfg);
 		from=linphone_proxy_config_get_identity(cfg);
+		if (cfg->op){
+			fixed_contact=sal_op_get_contact(cfg->op);
+			if (fixed_contact) {
+				ms_message("Contact for subscribe has been fixed using proxy to %s",fixed_contact);
+			}
+		}
 	}else from=linphone_core_get_primary_contact(fr->lc);
 	if (fr->outsub==NULL){
 		/* people for which we don't have yet an answer should appear as offline */
-		fr->status=LINPHONE_STATUS_OFFLINE;
+		fr->status=LinphoneStatusOffline;
 		/*
 		if (fr->lc->vtable.notify_recv)
 			fr->lc->vtable.notify_recv(fr->lc,(LinphoneFriend*)fr);
@@ -149,6 +136,7 @@ void __linphone_friend_do_subscribe(LinphoneFriend *fr){
 	}
 	fr->outsub=sal_op_new(fr->lc->sal);
 	sal_op_set_route(fr->outsub,route);
+	sal_op_set_contact(fr->outsub,fixed_contact);
 	sal_subscribe_presence(fr->outsub,from,friend);
 	fr->subscribe_active=TRUE;
 	ms_free(friend);
@@ -157,14 +145,19 @@ void __linphone_friend_do_subscribe(LinphoneFriend *fr){
 LinphoneFriend * linphone_friend_new(){
 	LinphoneFriend *obj=ms_new0(LinphoneFriend,1);
 	obj->pol=LinphoneSPAccept;
-	obj->status=LINPHONE_STATUS_OFFLINE;
+	obj->status=LinphoneStatusOffline;
 	obj->subscribe=TRUE;
 	return obj;	
 }
 
 LinphoneFriend *linphone_friend_new_with_addr(const char *addr){
+	LinphoneAddress* linphone_address = linphone_address_new(addr);
+	if (linphone_address == NULL) {
+		ms_error("Cannot create friend for address [%s]",addr?addr:"null");
+		return NULL;
+	}
 	LinphoneFriend *fr=linphone_friend_new();
-	if (linphone_friend_set_sip_addr(fr,addr)<0){
+	if (linphone_friend_set_addr(fr,linphone_address)<0){
 		linphone_friend_destroy(fr);
 		return NULL;
 	}
@@ -208,8 +201,8 @@ void linphone_core_interpret_friend_uri(LinphoneCore *lc, const char *uri, char
 	linphone_address_destroy(fr);
 }
 
-int linphone_friend_set_sip_addr(LinphoneFriend *lf, const char *addr){
-	LinphoneAddress *fr=linphone_address_new(addr);
+int linphone_friend_set_addr(LinphoneFriend *lf, const LinphoneAddress *addr){
+	LinphoneAddress *fr=linphone_address_clone(addr);
 	if (fr==NULL) {
 		ms_warning("Invalid friend sip uri: %s",addr);
 		return -1;
@@ -230,7 +223,7 @@ int linphone_friend_set_name(LinphoneFriend *lf, const char *name){
 	return 0;
 }
 
-int linphone_friend_send_subscribe(LinphoneFriend *fr, bool_t val){
+int linphone_friend_enable_subscribes(LinphoneFriend *fr, bool_t val){
 	fr->subscribe=val;
 	return 0;
 }
@@ -243,37 +236,37 @@ int linphone_friend_set_inc_subscribe_policy(LinphoneFriend *fr, LinphoneSubscri
 
 SalPresenceStatus linphone_online_status_to_sal(LinphoneOnlineStatus os){
 	switch(os){
-		case LINPHONE_STATUS_OFFLINE:
+		case LinphoneStatusOffline:
 			return SalPresenceOffline;
 		break;
-		case LINPHONE_STATUS_ONLINE:
+		case LinphoneStatusOnline:
 			return SalPresenceOnline;
 		break;
-		case LINPHONE_STATUS_BUSY:
+		case LinphoneStatusBusy:
 			return SalPresenceBusy;
 		break;
-		case LINPHONE_STATUS_BERIGHTBACK:
+		case LinphoneStatusBeRightBack:
 			return SalPresenceBerightback;
 		break;
-		case LINPHONE_STATUS_AWAY:
+		case LinphoneStatusAway:
 			return SalPresenceAway;
 		break;
-		case LINPHONE_STATUS_ONTHEPHONE:
+		case LinphoneStatusOnThePhone:
 			return SalPresenceOnthephone;
 		break;
-		case LINPHONE_STATUS_OUTTOLUNCH:
+		case LinphoneStatusOutToLunch:
 			return SalPresenceOuttolunch;
 		break;
-		case LINPHONE_STATUS_NOT_DISTURB:
+		case LinphoneStatusDoNotDisturb:
 			return SalPresenceDonotdisturb;
 		break;
-		case LINPHONE_STATUS_MOVED:
+		case LinphoneStatusMoved:
 			return SalPresenceMoved;
 		break;
-		case LINPHONE_STATUS_ALT_SERVICE:
+		case LinphoneStatusAltService:
 			return SalPresenceAltService;
 		break;
-		case LINPHONE_STATUS_PENDING:
+		case LinphoneStatusPending:
 			return SalPresenceOffline;
 		break;
 		default:
@@ -284,7 +277,9 @@ SalPresenceStatus linphone_online_status_to_sal(LinphoneOnlineStatus os){
 }
 
 void linphone_friend_notify(LinphoneFriend *lf, LinphoneOnlineStatus os){
-	//printf("Wish to notify %p, lf->nid=%i\n",lf,lf->nid);
+	char *addr=linphone_address_as_string(linphone_friend_get_address(lf));
+	ms_message("Want to notify %s, insub=%p",addr,lf->insub);
+	ms_free(addr);
 	if (lf->insub!=NULL){
 		sal_notify_presence(lf->insub,linphone_online_status_to_sal(os),NULL);
 	}
@@ -293,8 +288,6 @@ void linphone_friend_notify(LinphoneFriend *lf, LinphoneOnlineStatus os){
 static void linphone_friend_unsubscribe(LinphoneFriend *lf){
 	if (lf->outsub!=NULL) {
 		sal_unsubscribe(lf->outsub);
-		sal_op_release(lf->outsub);
-		lf->outsub=NULL;
 		lf->subscribe_active=FALSE;
 	}
 }
@@ -303,13 +296,19 @@ void linphone_friend_close_subscriptions(LinphoneFriend *lf){
 	linphone_friend_unsubscribe(lf);
 	if (lf->insub){
 		sal_notify_close(lf->insub);
-		sal_op_release(lf->insub);
-		lf->insub=NULL;
+		
 	}
 }
 
 void linphone_friend_destroy(LinphoneFriend *lf){
-	
+	if (lf->insub) {
+		sal_op_release(lf->insub);
+		lf->insub=NULL;
+	}
+	if (lf->outsub){
+		sal_op_release(lf->outsub);
+		lf->outsub=NULL;
+	}
 	if (lf->uri!=NULL) linphone_address_destroy(lf->uri);
 	if (lf->info!=NULL) buddy_info_free(lf->info);
 	ms_free(lf);
@@ -347,7 +346,7 @@ void linphone_friend_apply(LinphoneFriend *fr, LinphoneCore *lc){
 	if (fr->inc_subscribe_pending){
 		switch(fr->pol){
 			case LinphoneSPWait:
-				linphone_friend_notify(fr,LINPHONE_STATUS_PENDING);
+				linphone_friend_notify(fr,LinphoneStatusPending);
 				break;
 			case LinphoneSPAccept:
 				if (fr->lc!=NULL)
@@ -356,7 +355,7 @@ void linphone_friend_apply(LinphoneFriend *fr, LinphoneCore *lc){
 				  }
 				break;
 			case LinphoneSPDeny:
-				linphone_friend_notify(fr,LINPHONE_STATUS_OFFLINE);
+				linphone_friend_notify(fr,LinphoneStatusOffline);
 				break;
 		}
 		fr->inc_subscribe_pending=FALSE;
@@ -367,6 +366,7 @@ void linphone_friend_apply(LinphoneFriend *fr, LinphoneCore *lc){
 	}
 	ms_message("linphone_friend_apply() done.");
 	lc->bl_refresh=TRUE;
+	fr->commit=FALSE;
 }
 
 void linphone_friend_edit(LinphoneFriend *fr){
@@ -391,7 +391,8 @@ void linphone_core_add_friend(LinphoneCore *lc, LinphoneFriend *lf)
 		return ;
 	}
 	lc->friends=ms_list_append(lc->friends,lf);
-	linphone_friend_apply(lf,lc);
+	if ( linphone_core_ready(lc)) linphone_friend_apply(lf,lc);
+	else lf->commit=TRUE;
 	return ;
 }
 
@@ -404,6 +405,15 @@ void linphone_core_remove_friend(LinphoneCore *lc, LinphoneFriend* fl){
 	}
 }
 
+void linphone_core_send_initial_subscribes(LinphoneCore *lc){
+	const MSList *elem;
+	for(elem=lc->friends;elem!=NULL;elem=elem->next){
+		LinphoneFriend *f=(LinphoneFriend*)elem->data;
+		if (f->commit)
+			linphone_friend_apply(f,lc);
+	}
+}
+
 void linphone_friend_set_ref_key(LinphoneFriend *lf, const char *key){
 	if (lf->refkey!=NULL){
 		ms_free(lf->refkey);
@@ -562,7 +572,7 @@ void linphone_core_write_friends_config(LinphoneCore* lc)
 {
 	MSList *elem;
 	int i;
-	if (!lc->ready) return; /*dont write config when reading it !*/
+	if (! linphone_core_ready(lc)) return; /*dont write config when reading it !*/
 	for (elem=lc->friends,i=0; elem!=NULL; elem=ms_list_next(elem),i++){
 		linphone_friend_write_to_config_file(lc->config,(LinphoneFriend*)elem->data,i);
 	}
