@@ -110,7 +110,7 @@ void update_local_media_description(LinphoneCore *lc, LinphoneCall *call, SalMed
 }
 
 SalMediaDescription *create_local_media_description(LinphoneCore *lc, LinphoneCall *call){
-	unsigned int id=rand();
+	unsigned int id=rand() & 0xfff;
 	return _create_local_media_description(lc,call,id,id);
 }
 
@@ -1079,3 +1079,33 @@ bool_t linphone_call_echo_limiter_enabled(const LinphoneCall *call){
 	}
 }
 
+/**
+ * Returns the measured sound volume played locally (received from remote)
+ * It is expressed in dbm0. 
+**/
+float linphone_call_get_play_volume(LinphoneCall *call){
+	AudioStream *st=call->audiostream;
+	if (st && st->volsend){
+		float vol=0;
+		ms_filter_call_method(st->volsend,MS_VOLUME_GET,&vol);
+		return vol;
+		
+	}
+	return LINPHONE_VOLUME_DB_LOWEST;
+}
+
+/**
+ * Returns the measured sound volume recorded locally (sent to remote)
+ * It is expressed in dbm0. 
+**/
+float linphone_call_get_record_volume(LinphoneCall *call){
+	AudioStream *st=call->audiostream;
+	if (st && st->volrecv){
+		float vol=0;
+		ms_filter_call_method(st->volrecv,MS_VOLUME_GET,&vol);
+		return vol;
+		
+	}
+	return LINPHONE_VOLUME_DB_LOWEST;
+}
+
