@@ -177,7 +177,9 @@ void lp_config_parse(LpConfig *lpconfig, FILE *file){
 				if (sscanf(tmp,"%s",key)>0){
 
 					pos1++;
-					pos2=strchr(pos1,'\n');
+					pos2=strchr(pos1,'\r');
+					if (pos2==NULL)
+						pos2=strchr(pos1,'\n');
 					if (pos2==NULL) pos2=pos1+strlen(pos1);
 					else {
 						*pos2='\0'; /*replace the '\n' */
