@@ -345,52 +345,6 @@ bool_t lp_spawn_command_line_sync(const char *command, char **result,int *comman
 	return FALSE;
 }
 
-#if defined(HAVE_GETIFADDRS) && defined(INET6)
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <ifaddrs.h>
-bool_t host_has_ipv6_network()
-{
-	struct ifaddrs *ifp;
-	struct ifaddrs *ifpstart;
-	bool_t ipv6_present=FALSE;
-
-	if (getifaddrs (&ifpstart) < 0)
-	{
-		return FALSE;
-	}
-
-	for (ifp=ifpstart; ifp != NULL; ifp = ifp->ifa_next)
-	{
-		if (!ifp->ifa_addr)
-		  continue;
-
-		switch (ifp->ifa_addr->sa_family) {
-		case AF_INET:
-
-			break;
-		case AF_INET6:
-		    ipv6_present=TRUE;
-			break;
-		default:
-		        continue;
-  		}
-	}
-
-	freeifaddrs (ifpstart);
-
-	return ipv6_present;
-}
-#else
-
-bool_t host_has_ipv6_network()
-{
-	return FALSE;
-}
-
-
-#endif
-
 static ortp_socket_t create_socket(int local_port){
 	struct sockaddr_in laddr;
 	ortp_socket_t sock;
@@ -809,10 +763,10 @@ void _linphone_core_configure_resolver(){
 /*bionic declares _res but does not define nor export it !!*/
 #ifdef ANDROID
 	/*timeout and attempts are the same as retrans and retry, but are android specific names.*/
-	setenv("RES_OPTIONS","timeout:1 attempts:2 retrans:1 retry:2",1);
+	setenv("RES_OPTIONS","timeout:2 attempts:2 retrans:2 retry:2",1);
 #else
 	res_init();
-	_res.retrans=1; /*retransmit every second*/
+	_res.retrans=2; /*retransmit every two seconds*/
 	_res.retry=2; /*only two times per DNS server*/
 #endif
 }
