@@ -1,6 +1,6 @@
 /*
 mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2006  Simon MORLAT (simon.morlat@linphone.org)
+Copyright (C) 2006-2010  Belledonne Communications SARL (simon.morlat@linphone.org)
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -38,7 +38,7 @@ static void yuv_buf_init(YuvBuf *buf, int w, int h, uint8_t *ptr){
 	buf->strides[3]=0;
 }
 
-int yuv_buf_init_from_mblk(YuvBuf *buf, mblk_t *m){
+int ms_yuv_buf_init_from_mblk(YuvBuf *buf, mblk_t *m){
 	int size=m->b_wptr-m->b_rptr;
 	int w,h;
 	if (size==(MS_VIDEO_SIZE_QCIF_W*MS_VIDEO_SIZE_QCIF_H*3)/2){
@@ -116,6 +116,9 @@ int yuv_buf_init_from_mblk(YuvBuf *buf, mblk_t *m){
 	}else if (size==(MS_VIDEO_SIZE_WQCIF_W*MS_VIDEO_SIZE_WQCIF_H*3)/2){
 		w=MS_VIDEO_SIZE_WQCIF_W;
 		h=MS_VIDEO_SIZE_WQCIF_H;
+	}else if (size==(MS_VIDEO_SIZE_CVD_W*MS_VIDEO_SIZE_CVD_H*3)/2){
+		w=MS_VIDEO_SIZE_CVD_W;
+		h=MS_VIDEO_SIZE_CVD_H;
 	}else if (size==(160*112*3)/2){/*format used by econf*/
 		w=160;
 		h=112;
@@ -126,19 +129,58 @@ int yuv_buf_init_from_mblk(YuvBuf *buf, mblk_t *m){
 		ms_error("Unsupported image size: size=%i (bug somewhere !)",size);
 		return -1;
 	}
+	if (mblk_get_video_orientation(m)==MS_VIDEO_PORTRAIT){
+		int tmp=h;
+		h=w;
+		w=tmp;
+	}
 	yuv_buf_init(buf,w,h,m->b_rptr);
 	return 0;
 }
 
-void yuv_buf_init_from_mblk_with_size(YuvBuf *buf, mblk_t *m, int w, int h){
-	yuv_buf_init(buf,w,h,m->b_rptr);
+int ms_yuv_buf_init_from_mblk_with_size(YuvBuf *buf, mblk_t *m, int w, int h){
+  yuv_buf_init(buf,w,h,m->b_rptr);
+  return 0;
+
 }
 
-mblk_t * yuv_buf_alloc(YuvBuf *buf, int w, int h){
+int ms_picture_init_from_mblk_with_size(MSPicture *buf, mblk_t *m, MSPixFmt fmt, int w, int h){
+	switch(fmt){
+		case MS_YUV420P:
+			return ms_yuv_buf_init_from_mblk_with_size(buf,m,w,h);
+		break;
+		case MS_YUY2:
+		case MS_YUYV:
+			memset(buf,0,sizeof(*buf));
+			buf->w=w;
+			buf->h=h;
+			buf->planes[0]=m->b_rptr;
+			buf->strides[0]=w*2;
+		break;
+		case MS_RGB24:
+		case MS_RGB24_REV:
+			memset(buf,0,sizeof(*buf));
+			buf->w=w;
+			buf->h=h;
+			buf->planes[0]=m->b_rptr;
+			buf->strides[0]=w*3;
+		break;
+		default:
+			ms_fatal("FIXME: unsupported format %i",fmt);
+			return -1;
+	}
+	return 0;
+}
+
+mblk_t * ms_yuv_buf_alloc(YuvBuf *buf, int w, int h){
 	int size=(w*h*3)/2;
 	const int padding=16;
 	mblk_t *msg=allocb(size+padding,0);
 	yuv_buf_init(buf,w,h,msg->b_wptr);
+	if (h>w)
+		mblk_set_video_orientation(msg,MS_VIDEO_PORTRAIT);
+	else
+		mblk_set_video_orientation(msg,MS_VIDEO_LANDSCAPE);
 	msg->b_wptr+=size;
 	return msg;
 }
@@ -153,7 +195,7 @@ static void plane_copy(const uint8_t *src_plane, int src_stride,
 	}
 }
 
-static void yuv_buf_copy(uint8_t *src_planes[], const int src_strides[], 
+void ms_yuv_buf_copy(uint8_t *src_planes[], const int src_strides[], 
 		uint8_t *dst_planes[], const int dst_strides[3], MSVideoSize roi){
 	plane_copy(src_planes[0],src_strides[0],dst_planes[0],dst_strides[0],roi);
 	roi.width=roi.width/2;
@@ -176,7 +218,7 @@ static void plane_mirror(uint8_t *p, int linesize, int w, int h){
 }
 
 /*in place mirroring*/
-static void yuv_buf_mirror(YuvBuf *buf){
+void ms_yuv_buf_mirror(YuvBuf *buf){
 	plane_mirror(buf->planes[0],buf->strides[0],buf->w,buf->h);
 	plane_mirror(buf->planes[1],buf->strides[1],buf->w/2,buf->h/2);
 	plane_mirror(buf->planes[2],buf->strides[2],buf->w/2,buf->h/2);
@@ -287,66 +329,227 @@ MSVideoSize ms_video_size_get_just_lower_than(MSVideoSize vs){
 	return ret;
 }
 
+void ms_rgb_to_yuv(const uint8_t rgb[3], uint8_t yuv[3]){
+	yuv[0]=(uint8_t)(0.257*rgb[0] + 0.504*rgb[1] + 0.098*rgb[2] + 16);
+	yuv[1]=(uint8_t)(-0.148*rgb[0] - 0.291*rgb[1] + 0.439*rgb[2] + 128);
+	yuv[2]=(uint8_t)(0.439*rgb[0] - 0.368*rgb[1] - 0.071*rgb[2] + 128);
+}
+
 #if !defined(NO_FFMPEG)
 
-struct ms_swscaleDesc ms_swscale_desc = {
-	NULL,
-	NULL,
-	NULL,
-	NULL
+
+int ms_pix_fmt_to_ffmpeg(MSPixFmt fmt){
+	switch(fmt){
+		case MS_RGBA32:
+			return PIX_FMT_RGBA;
+		case MS_RGB24:
+			return PIX_FMT_RGB24;
+		case MS_RGB24_REV:
+			return PIX_FMT_BGR24;
+		case MS_YUV420P:
+			return PIX_FMT_YUV420P;
+		case MS_YUYV:
+			return PIX_FMT_YUYV422;
+		case MS_UYVY:
+			return PIX_FMT_UYVY422;
+		case MS_YUY2:
+			return PIX_FMT_YUYV422;   /* <- same as MS_YUYV */
+		case MS_RGB565:
+			return PIX_FMT_RGB565;
+		default:
+			ms_fatal("format not supported.");
+			return -1;
+	}
+	return -1;
+}
+
+MSPixFmt ffmpeg_pix_fmt_to_ms(int fmt){
+	switch(fmt){
+		case PIX_FMT_RGB24:
+			return MS_RGB24;
+		case PIX_FMT_BGR24:
+			return MS_RGB24_REV;
+		case PIX_FMT_YUV420P:
+			return MS_YUV420P;
+		case PIX_FMT_YUYV422:
+			return MS_YUYV;     /* same as MS_YUY2 */
+		case PIX_FMT_UYVY422:
+			return MS_UYVY;
+		case PIX_FMT_RGBA:
+			return MS_RGBA32;
+		case PIX_FMT_RGB565:
+			return MS_RGB565;
+		default:
+			ms_fatal("format not supported.");
+			return MS_YUV420P; /* default */
+	}
+	return MS_YUV420P; /* default */
+}
+
+struct _MSFFScalerContext{
+	struct SwsContext *ctx;
+	int src_h;
 };
 
-struct ms_SwsContext *ms_sws_getContext(int srcW, int srcH, int srcFormat,
-                                  int dstW, int dstH, int dstFormat,
-                                  int flags, struct _SwsFilter *srcFilter,
-                                  struct _SwsFilter *dstFilter, double *param)
-{
-	if (ms_swscale_desc.sws_getContext==NULL)
-	{
-		ms_swscale_desc.sws_getContext=(sws_getContextFunc)sws_getContext;
-		ms_swscale_desc.sws_freeContext=(sws_freeContextFunc)sws_freeContext;
-		ms_swscale_desc.sws_scale=(sws_scaleFunc)sws_scale;
+typedef struct _MSFFScalerContext MSFFScalerContext;
+
+static MSScalerContext *ff_create_swscale_context(int src_w, int src_h, MSPixFmt src_fmt,
+                                          int dst_w, int dst_h, MSPixFmt dst_fmt, int flags){
+	int ff_flags=0;
+	MSFFScalerContext *ctx=ms_new(MSFFScalerContext,1);
+	ctx->src_h=src_h;
+	if (flags & MS_SCALER_METHOD_BILINEAR)
+		ff_flags|=SWS_BILINEAR;
+	else if (flags & MS_SCALER_METHOD_NEIGHBOUR)
+		ff_flags|=SWS_BILINEAR;
+	ctx->ctx=sws_getContext (src_w,src_h,ms_pix_fmt_to_ffmpeg (src_fmt),
+	                                       dst_w,dst_h,ms_pix_fmt_to_ffmpeg (dst_fmt),ff_flags,NULL,NULL,NULL);
+	if (ctx->ctx==NULL){
+		ms_free(ctx);
+		ctx=NULL;
 	}
-	return (struct ms_SwsContext *)ms_swscale_desc.sws_getContext(srcW, srcH, srcFormat, dstW, dstH, dstFormat,
-		flags, srcFilter, dstFilter, param);
+	return (MSScalerContext*)ctx;
 }
 
-void ms_sws_freeContext(struct ms_SwsContext *swsContext)
-{
-	ms_swscale_desc.sws_freeContext(swsContext);
+static int ff_sws_scale(MSScalerContext *ctx, uint8_t *src[], int src_strides[], uint8_t *dst[], int dst_strides[]){
+	MSFFScalerContext *fctx=(MSFFScalerContext*)ctx;
+	int err=sws_scale(fctx->ctx,(const uint8_t * const*)src,src_strides,0,fctx->src_h,dst,dst_strides);
+	if (err<0) return -1;
+	return 0;
 }
 
-int ms_sws_scale(struct ms_SwsContext *context, uint8_t* srcSlice[], int srcStride[],
-              int srcSliceY, int srcSliceH, uint8_t* dst[], int dstStride[])
-{
-	return ms_swscale_desc.sws_scale(context, srcSlice, srcStride, srcSliceY, srcSliceH, dst, dstStride);
+static void ff_sws_free(MSScalerContext *ctx){
+	MSFFScalerContext *fctx=(MSFFScalerContext*)ctx;
+	if (fctx->ctx) sws_freeContext(fctx->ctx);
+	ms_free(ctx);
 }
 
-void ms_yuv_buf_mirror(YuvBuf *buf)
-{
-	if (ms_swscale_desc.yuv_buf_mirror==NULL)
-	{
-		ms_swscale_desc.yuv_buf_mirror=(yuv_buf_mirrorFunc)yuv_buf_mirror;
-	}
-	return ms_swscale_desc.yuv_buf_mirror(buf);
-}
-
-void ms_yuv_buf_copy(uint8_t *src_planes[], const int src_strides[], 
-		uint8_t *dst_planes[], const int dst_strides[3], MSVideoSize roi)
-{
-	if (ms_swscale_desc.yuv_buf_copy==NULL)
-	{
-		ms_swscale_desc.yuv_buf_copy=(yuv_buf_copyFunc)yuv_buf_copy;
-	}
-	return ms_swscale_desc.yuv_buf_copy(src_planes, src_strides, dst_planes, dst_strides, roi);
-}
-
-void ms_video_set_video_func(struct ms_swscaleDesc *_ms_swscale_desc)
-{
-	ms_swscale_desc.sws_getContext=_ms_swscale_desc->sws_getContext;
-	ms_swscale_desc.sws_freeContext=_ms_swscale_desc->sws_freeContext;
-	ms_swscale_desc.sws_scale=_ms_swscale_desc->sws_scale;
-	ms_swscale_desc.yuv_buf_mirror=_ms_swscale_desc->yuv_buf_mirror;
-}
+static MSScalerDesc ffmpeg_scaler={
+	ff_create_swscale_context,
+	ff_sws_scale,
+	ff_sws_free
+};
 
 #endif
+
+#if 0
+
+/*
+We use openmax-dl (from ARM) to optimize some scaling routines.
+*/
+
+#include "omxIP.h"
+
+typedef struct AndroidScalerCtx{
+	MSFFScalerContext base;
+	OMXIPColorSpace cs;
+	OMXSize src_size;
+	OMXSize dst_size;
+	bool_t use_omx;
+}AndroidScalerCtx;
+
+/* for android we use ffmpeg's scaler except for YUV420P-->RGB565, for which we prefer 
+ another arm neon optimized routine */
+
+static MSScalerContext *android_create_scaler_context(int src_w, int src_h, MSPixFmt src_fmt,
+                                          int dst_w, int dst_h, MSPixFmt dst_fmt, int flags){
+	AndroidScalerCtx *ctx=ms_new0(AndroidScalerCtx,1);
+	if (src_fmt==MS_YUV420P && dst_fmt==MS_RGB565){
+		ctx->use_omx=TRUE;
+		ctx->cs=OMX_IP_BGR565;
+		ctx->src_size.width=src_w;
+		ctx->src_size.height=src_h;
+		ctx->dst_size.width=dst_w;
+		ctx->dst_size.height=dst_h;
+	}else{
+		unsigned int ff_flags=0;
+		ctx->base.src_h=src_h;
+		if (flags & MS_SCALER_METHOD_BILINEAR)
+			ff_flags|=SWS_BILINEAR;
+		else if (flags & MS_SCALER_METHOD_NEIGHBOUR)
+			ff_flags|=SWS_BILINEAR;
+		ctx->base.ctx=sws_getContext (src_w,src_h,ms_pix_fmt_to_ffmpeg (src_fmt),
+	                                       dst_w,dst_h,ms_pix_fmt_to_ffmpeg (dst_fmt),ff_flags,NULL,NULL,NULL);
+		if (ctx->base.ctx==NULL){
+			ms_free(ctx);
+			ctx=NULL;
+		}
+	}
+	return (MSScalerContext *)ctx;
+}
+
+static int android_scaler_process(MSScalerContext *ctx, uint8_t *src[], int src_strides[], uint8_t *dst[], int dst_strides[]){
+	AndroidScalerCtx *actx=(AndroidScalerCtx*)ctx;
+	if (actx->use_omx){
+		int ret;
+		OMX_U8 *osrc[3];
+		OMX_INT osrc_strides[3];
+		OMX_INT xrr_max;
+		OMX_INT yrr_max;
+
+		osrc[0]=src[0];
+		osrc[1]=src[1];
+		osrc[2]=src[2];
+		osrc_strides[0]=src_strides[0];
+		osrc_strides[1]=src_strides[1];
+		osrc_strides[2]=src_strides[2];
+
+		xrr_max = (OMX_INT) ((( (OMX_F32) ((actx->src_size.width&~1)-1) / ((actx->dst_size.width&~1)-1))) * (1<<16) +0.5);
+		yrr_max = (OMX_INT) ((( (OMX_F32) ((actx->src_size.height&~1)-1) / ((actx->dst_size.height&~1)-1))) * (1<< 16) +0.5);
+
+		ret=omxIPCS_YCbCr420RszCscRotBGR_U8_P3C3R((const OMX_U8**)osrc,osrc_strides,actx->src_size,dst[0],dst_strides[0],actx->dst_size,actx->cs,
+				OMX_IP_BILINEAR, OMX_IP_DISABLE, xrr_max,yrr_max);
+		if (ret!=OMX_Sts_NoErr){
+			ms_error("omxIPCS_YCbCr420RszCscRotBGR_U8_P3C3R() failed : %i",ret);
+			return -1;
+		}
+		return 0;
+	}
+	return ff_sws_scale(ctx,src,src_strides,dst,dst_strides);
+}
+
+static void android_scaler_free(MSScalerContext *ctx){
+	ff_sws_free(ctx);
+}
+
+static MSScalerDesc android_scaler={
+	android_create_scaler_context,
+	android_scaler_process,
+	android_scaler_free
+};
+
+#endif
+
+
+#ifdef ANDROID
+
+extern MSScalerDesc ms_android_scaler;
+
+static MSScalerDesc *scaler_impl=&ms_android_scaler;
+#elif !defined(NO_FFMPEG)
+static MSScalerDesc *scaler_impl=&ffmpeg_scaler;
+#else
+static MSScalerDesc *scaler_impl=NULL;
+#endif
+
+MSScalerContext *ms_scaler_create_context(int src_w, int src_h, MSPixFmt src_fmt,
+                                          int dst_w, int dst_h, MSPixFmt dst_fmt, int flags){
+	if (scaler_impl)
+		return scaler_impl->create_context(src_w,src_h,src_fmt,dst_w,dst_h,dst_fmt, flags);
+	ms_fatal("No scaler implementation built-in, please supply one with ms_video_set_scaler_impl ()");
+	return NULL;
+}
+
+int ms_scaler_process(MSScalerContext *ctx, uint8_t *src[], int src_strides[], uint8_t *dst[], int dst_strides[]){
+	return scaler_impl->context_process(ctx,src,src_strides,dst,dst_strides);
+}
+
+void ms_scaler_context_free(MSScalerContext *ctx){
+	scaler_impl->context_free(ctx);
+}
+
+void ms_video_set_scaler_impl(MSScalerDesc *desc){
+	scaler_impl=desc;
+}
+
+
