@@ -45,13 +45,14 @@ static int cond=1;
 
 static const char * capture_card=NULL;
 static const char * playback_card=NULL;
+static const char * camera=NULL;
 static const char *infile=NULL,*outfile=NULL;
 static float ng_threshold=-1;
 static bool_t use_ng=FALSE;
 static bool_t two_windows=FALSE;
 
 /* starting values echo canceller */
-static int ec_len_ms=0, ec_delay_ms=250, ec_framesize;
+static int ec_len_ms=0, ec_delay_ms=0, ec_framesize=0;
 
 
 static void stop_handler(int signum)
@@ -141,13 +142,16 @@ const char *usage="mediastream --local <port> --remote <ip:port> --payload <payl
 								"[ --height <pixels> ]\n"
 								"[ --bitrate <bits per seconds>]\n"
 								"[ --ec (enable echo canceller)]\n"
+								"[ --ec-tail <echo canceller tail length in ms> ]\n"
+								"[ --ec-delay <echo canceller delay in ms> ]\n"
 								"[ --agc (enable automatic gain control)]\n"
 								"[ --ng (enable noise gate)]\n"
 								"[ --ng-threshold <(float) [0-1]> (noise gate threshold)]\n"
 								"[ --capture-card <name>] \n"
 								"[ --playback-card <name>] \n"
 								"[ --infile	<input wav file>] specify a wav file to be used for input, instead of soundcard\n"
-								"[ --outfile <output wav file>] specify a wav file to write audio into, instead of soundcard\n";
+								"[ --outfile <output wav file>] specify a wav file to write audio into, instead of soundcard\n"
+								"[ --camera <camera id as listed at startup> ]\n";
 
 static void run_media_streams(int localport, const char *remote_ip, int remoteport, int payload, const char *fmtp,
           int jitter, int bitrate, MSVideoSize vs, bool_t ec, bool_t agc, bool_t eq);
@@ -225,6 +229,12 @@ int main(int argc, char * argv[])
 			playback_card=argv[i];
 		}else if (strcmp(argv[i],"--ec")==0){
 			ec=TRUE;
+		}else if (strcmp(argv[i],"--ec-tail")==0){
+			i++;
+			ec_len_ms=atoi(argv[i]);
+		}else if (strcmp(argv[i],"--ec-delay")==0){
+			i++;
+			ec_delay_ms=atoi(argv[i]);
 		}else if (strcmp(argv[i],"--agc")==0){
 			agc=TRUE;
 		}else if (strcmp(argv[i],"--eq")==0){
@@ -242,6 +252,9 @@ int main(int argc, char * argv[])
 		}else if (strcmp(argv[i],"--outfile")==0){
 			i++;
 			outfile=argv[i];
+		}else if (strcmp(argv[i],"--camera")==0){
+			i++;
+			camera=argv[i];
 		}
 	}
 
@@ -255,6 +268,7 @@ static void run_media_streams(int localport, const char *remote_ip, int remotepo
 	AudioStream *audio=NULL;
 #ifdef VIDEO_ENABLED
 	VideoStream *video=NULL;
+	MSWebCam *cam=NULL;
 #endif
 	RtpSession *session=NULL;
 	PayloadType *pt;
@@ -301,12 +315,17 @@ static void run_media_streams(int localport, const char *remote_ip, int remotepo
 		video=video_stream_new(localport, ms_is_ipv6(remote_ip));
 		video_stream_set_sent_video_size(video,vs);
 		video_stream_use_preview_video_window(video,two_windows);
+		
+		if (camera)
+			cam=ms_web_cam_manager_get_cam(ms_web_cam_manager_get(),camera);
+		if (cam==NULL)
+			cam=ms_web_cam_manager_get_default_cam(ms_web_cam_manager_get());
 		video_stream_start(video,profile,
 					remote_ip,
 					remoteport,remoteport+1,
 					payload,
-					jitter,
-					ms_web_cam_manager_get_default_cam(ms_web_cam_manager_get()));
+					jitter,cam
+					);
 		session=video->session;
 #else
 		printf("Error: video support not compiled.\n");
