@@ -31,6 +31,16 @@ esac
 if test -d /opt/local/share/aclocal ; then
         ACLOCAL_ARGS="-I /opt/local/share/aclocal"
 fi
+if test -d /opt/local/share/aclocal ; then
+	ACLOCAL_ARGS="-I /opt/local/share/aclocal"
+fi
+if test -d /usr/local/share/aclocal ; then
+	ACLOCAL_ARGS="$ACLOCAL_ARGS -I /usr/local/share/aclocal"
+fi
+
+if test -d /share/aclocal ; then
+        ACLOCAL_ARGS="$ACLOCAL_ARGS -I /share/aclocal"
+fi
 
 
 set -x
