@@ -17,12 +17,55 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+/** \mainpage oRTP API documentation
+ *
+ * \section init Initializing oRTP
+ *
+ * see ortp.h documentation.
+ *
+ * \section rtpsession the RtpSession object
+ *
+ * see the rtpsession.h documentation.
+ *
+ * \section payloadtypes Managing PayloadType(s) and RtpProfile(s)
+ *
+ * see the payloadtype.h documentation.
+ *
+ * \section telephonevents Sending and receiving telephone-event (RFC2833)
+ *
+ * see the telephonyevents.h documentation.
+ * To get informed about incoming telephone-event you can register a callback
+ * using rtp_session_signal_connect() or by registering an event queue using
+ * rtp_session_register_event_queue().
+ *
+ * \section sessionset Managing several RtpSession simultaneously
+ *
+ * see the sessionset.h documentation.
+ *
+ * \section rtcp Parsing incoming rtcp packets.
+ *
+ * The parsing api is defined in rtcp.h (not yet documented).
+ *
+ * \section examples Examples
+ *
+ * oRTP comes with a set of examples in src/tests.
+ * - rtprecv.c rtpsend.c show how to receive and send a single RTP stream.
+ * - mrtprecv.c mrtpsend.c show how to receive and send multiple RTP streams
+ *   simultaneously
+ * 
+ */
+
+/** 
+ * \file ortp.h
+ * \brief General purpose library functions.
+ *
+**/
 
 #ifndef ORTP_H
 #define ORTP_H
 
-#include <ortp/rtpsession.h>
-#include <ortp/sessionset.h>
+#include "ortp/rtpsession.h"
+#include "ortp/sessionset.h"
 
 #ifdef __cplusplus
 extern "C"
@@ -72,13 +115,6 @@ void ortp_logv(int level, const char *fmt, va_list args);
 
 void ortp_set_log_level_mask(int levelmask);
 
-static inline void ortp_log(OrtpLogLevel lev, const char *fmt,...){
-	va_list args;
-	va_start (args, fmt);
-	ortp_logv(lev, fmt, args);
- 	va_end (args);
-}
-
 #ifdef ORTP_DEBUG_MODE
 static inline void ortp_debug(const char *fmt,...)
 {
@@ -88,48 +124,58 @@ static inline void ortp_debug(const char *fmt,...)
   va_end (args);
 }
 #else
-#if defined(WIN32) || defined(_WIN32_WCE)
-static inline void ortp_debug(const char *fmt,...)
-{
-  va_list args;
-  va_start (args, fmt);
-  va_end (args);
-}
-#else
+
 #define ortp_debug(...)
+
 #endif
-#endif
+
+#ifdef ORTP_NOMESSAGE_MODE
+
+#define ortp_log(...)
+#define ortp_message(...)
+#define ortp_warning(...)
+
+#else
+
+static inline void ortp_log(OrtpLogLevel lev, const char *fmt,...){
+	va_list args;
+	va_start (args, fmt);
+	ortp_logv(lev, fmt, args);
+ 	va_end (args);
+}
 
 static inline void ortp_message(const char *fmt,...)
 {
-  va_list args;
-  va_start (args, fmt);
-  ortp_logv(ORTP_MESSAGE, fmt, args);
-  va_end (args);
+	va_list args;
+	va_start (args, fmt);
+	ortp_logv(ORTP_MESSAGE, fmt, args);
+	va_end (args);
 }
 
 static inline void ortp_warning(const char *fmt,...)
 {
-  va_list args;
-  va_start (args, fmt);
-  ortp_logv(ORTP_WARNING, fmt, args);
-  va_end (args);
+	va_list args;
+	va_start (args, fmt);
+	ortp_logv(ORTP_WARNING, fmt, args);
+	va_end (args);
 }
 
+#endif
+
 static inline void ortp_error(const char *fmt,...)
 {
-  va_list args;
-  va_start (args, fmt);
-  ortp_logv(ORTP_ERROR, fmt, args);
-  va_end (args);
+	va_list args;
+	va_start (args, fmt);
+	ortp_logv(ORTP_ERROR, fmt, args);
+	va_end (args);
 }
 
 static inline void ortp_fatal(const char *fmt,...)
 {
-  va_list args;
-  va_start (args, fmt);
-  ortp_logv(ORTP_FATAL, fmt, args);
-  va_end (args);
+	va_list args;
+	va_start (args, fmt);
+	ortp_logv(ORTP_FATAL, fmt, args);
+	va_end (args);
 }
 
 
